#include "stdafx.h"
#include "ButtonFont.h"

NameValuePair alignValues[] = {
		{ "left", DT_LEFT },
		{ "center", DT_CENTER },
		{ "right", DT_RIGHT },
		{ 0, 0 }
		};

NameValuePair vertAlignValues[] = {
		{ "top", DT_TOP },
		{ "center", DT_VCENTER },
		{ "bottom", DT_BOTTOM },
		{ 0, 0 }
		};

ButtonFont::ButtonFont(const string &name)
{
	normalFont = 0;
	activeFont = 0;
	minimizedFont = 0;
	flashingFont = 0;

	defaultfont.leftBorder = GetxRCInt(name, "FontLeftBorder", 0, 0);
	defaultfont.rightBorder = GetxRCInt(name, "FontRightBorder", 0, 0);
	defaultfont.topBorder = GetxRCInt(name, "FontTopBorder", 0, 0);
	defaultfont.bottomBorder = GetxRCInt(name, "FontBottomBorder", 0, 0);

	defaultfont.shadow = GetxRCBool(name, "FontShadow", false);
	defaultfont.shadowcolor = GetxRCColor(name, "FontShadowColor", RGB(128, 128, 128));
	defaultfont.shadowX = GetxRCInt(name, "FontShadowX", 1);
	defaultfont.shadowY = GetxRCInt(name, "FontShadowY", 1);

	defaultfont.textoutline = GetxRCBool(name, "FontOutline", false);
	defaultfont.textoutlinecolor = GetxRCColor(name, "FontOutlineColor", RGB(128, 128, 128));

	defaultfont.name = GetxRCString(name, "Font", "Arial");
	defaultfont.height = GetxRCInt(name, "FontHeight", 15, 0);
	defaultfont.color = GetxRCColor(name, "FontColor", RGB(0, 0, 0));

	defaultfont.bold = GetxRCBool(name, "FontBold", false);
	defaultfont.italic = GetxRCBool(name, "FontItalic", false);
	defaultfont.underline = GetxRCBool(name, "FontUnderline", false);

	defaultfont.align = GetxRCNamedValue(name, "FontAlign", alignValues, DT_LEFT);
	defaultfont.vertalign = GetxRCNamedValue(name, "FontVertAlign", vertAlignValues, DT_VCENTER);
}

ButtonFont::~ButtonFont()
{
	if(normalFont)
		DeleteObject(normalFont);
	if(activeFont)
		DeleteObject(activeFont);
	if(minimizedFont)
		DeleteObject(minimizedFont);
	if(flashingFont)
		DeleteObject(flashingFont);
}

void ButtonFont::subconfigure(string type, FONTSETTINGS &workSettings, FONTSETTINGS &defaultSettings)
{
	workSettings.leftBorder = GetxRCInt(type, "FontLeftBorder", defaultSettings.leftBorder, 0);
	workSettings.rightBorder = GetxRCInt(type, "FontRightBorder", defaultSettings.rightBorder, 0);
	workSettings.topBorder = GetxRCInt(type, "FontTopBorder", defaultSettings.topBorder, 0);
	workSettings.bottomBorder = GetxRCInt(type, "FontBottomBorder", defaultSettings.bottomBorder, 0);

	workSettings.shadow = GetxRCBool(type, "FontShadow", defaultSettings.shadow);
	workSettings.shadowcolor = GetxRCColor(type, "FontShadowColor", defaultSettings.shadowcolor);
	workSettings.shadowX = GetxRCInt(type, "FontShadowX", defaultSettings.shadowX);
	workSettings.shadowY = GetxRCInt(type, "FontShadowY", defaultSettings.shadowY);

	workSettings.textoutline = GetxRCBool(type, "FontOutline", defaultSettings.textoutline);
	workSettings.textoutlinecolor = GetxRCColor(type, "FontOutlineColor", defaultSettings.textoutlinecolor);

	workSettings.name = GetxRCString(type, "Font", defaultSettings.name);
	workSettings.height = GetxRCInt(type, "FontHeight", defaultSettings.height, 0);
	workSettings.color = GetxRCColor(type, "FontColor", defaultSettings.color);

	workSettings.bold = GetxRCBool(type, "FontBold", defaultSettings.bold);
	workSettings.italic = GetxRCBool(type, "FontItalic", defaultSettings.italic);
	workSettings.underline = GetxRCBool(type, "FontUnderline", defaultSettings.underline);

	workSettings.align = GetxRCNamedValue(type, "FontAlign", alignValues, defaultSettings.align);
	workSettings.vertalign = GetxRCNamedValue(type, "FontVertAlign", vertAlignValues, defaultSettings.vertalign);
}

void ButtonFont::configure(const string &name)
{
	subconfigure((name + "Normal"), normal, defaultfont);
	createHandle(normal, normalFont);

	subconfigure((name + "Active"), active, normal);
	createHandle(active, activeFont);

	subconfigure((name + "Minimized"), minimized, normal);
	createHandle(minimized, minimizedFont);

	subconfigure((name + "Flashing"), flashing, normal);
	createHandle(flashing, flashingFont);
}

void ButtonFont::apply(HDC hDC, int x, int y, int width, int height, const string &caption, int buttonType)
{
	RECT r;

	int textColor;
	int vertAlign;
	int align;

	int bkMode = SetBkMode(hDC, TRANSPARENT);

	if ( buttonType == 0 )
	{
		r.left = x + normal.leftBorder;
		r.top = y + normal.topBorder;
		r.right = x + width - normal.rightBorder;
		r.bottom = y + height - normal.bottomBorder;

		normalFont = (HFONT) SelectObject(hDC, normalFont);
		textColor = SetTextColor(hDC, normal.color);
		vertAlign = normal.vertalign;
		align = normal.align;

		if (normal.shadow && !normal.textoutline)
		{
			OffsetRect(&r, (normal.shadowX + 1) / 2, (normal.shadowY + 1) / 2);
			SetTextColor(hDC, normal.shadowcolor);
			DrawTextEx(hDC, (char *) caption.c_str(), caption.length(), &r, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | DT_END_ELLIPSIS, 0);
			OffsetRect(&r, -normal.shadowX, -normal.shadowY);
			SetTextColor(hDC, normal.color);
		}
		if (normal.textoutline)
		{
			SetTextColor(hDC, normal.textoutlinecolor);
			drawOutline(hDC, r, align, vertAlign, caption);
			SetTextColor(hDC, normal.color);
		}
	}
	else if ( buttonType == 1 )
	{
		r.left = x + active.leftBorder;
		r.top = y + active.topBorder;
		r.right = x + width - active.rightBorder;
		r.bottom = y + height - active.bottomBorder;

		activeFont = (HFONT) SelectObject(hDC, activeFont);
		textColor = SetTextColor(hDC, active.color);
		vertAlign = active.vertalign;
		align = active.align;

		if (active.shadow && !active.textoutline)
		{
			OffsetRect(&r, (active.shadowX + 1) / 2, (active.shadowY + 1) / 2);
			SetTextColor(hDC, active.shadowcolor);
			DrawTextEx(hDC, (char *) caption.c_str(), caption.length(), &r, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | DT_END_ELLIPSIS, 0);
			OffsetRect(&r, -active.shadowX, -active.shadowY);
			SetTextColor(hDC, active.color);
		}
		if (active.textoutline)
		{
			SetTextColor(hDC, active.textoutlinecolor);
			drawOutline(hDC, r, align, vertAlign, caption);
			SetTextColor(hDC, active.color);
		}
	}
	else if ( buttonType == 2 )
	{
		r.left = x + minimized.leftBorder;
		r.top = y + minimized.topBorder;
		r.right = x + width - minimized.rightBorder;
		r.bottom = y + height - minimized.bottomBorder;

		minimizedFont = (HFONT) SelectObject(hDC, minimizedFont);
		textColor = SetTextColor(hDC, minimized.color);
		vertAlign = minimized.vertalign;
		align = minimized.align;

		if (minimized.shadow && !minimized.textoutline)
		{
			OffsetRect(&r, (minimized.shadowX + 1) / 2, (minimized.shadowY + 1) / 2);
			SetTextColor(hDC, minimized.shadowcolor);
			DrawTextEx(hDC, (char *) caption.c_str(), caption.length(), &r, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | DT_END_ELLIPSIS, 0);
			OffsetRect(&r, -minimized.shadowX, -minimized.shadowY);
			SetTextColor(hDC, minimized.color);
		}
		if (minimized.textoutline)
		{
			SetTextColor(hDC, minimized.textoutlinecolor);
			drawOutline(hDC, r, align, vertAlign, caption);
			SetTextColor(hDC, minimized.color);
		}
	}
	else if ( buttonType == 3 )
	{
		r.left = x + flashing.leftBorder;
		r.top = y + flashing.topBorder;
		r.right = x + width - flashing.rightBorder;
		r.bottom = y + height - flashing.bottomBorder;

		flashingFont = (HFONT) SelectObject(hDC, flashingFont);
		textColor = SetTextColor(hDC, flashing.color);
		vertAlign = flashing.vertalign;
		align = flashing.align;

		if (flashing.shadow && !flashing.textoutline)
		{
			OffsetRect(&r, (flashing.shadowX + 1) / 2, (flashing.shadowY + 1) / 2);
			SetTextColor(hDC, flashing.shadowcolor);
			DrawTextEx(hDC, (char *) caption.c_str(), caption.length(), &r, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | DT_END_ELLIPSIS, 0);
			OffsetRect(&r, -flashing.shadowX, -flashing.shadowY);
			SetTextColor(hDC, flashing.color);
		}
		if (flashing.textoutline)
		{
			SetTextColor(hDC, flashing.textoutlinecolor);
			drawOutline(hDC, r, align, vertAlign, caption);
			SetTextColor(hDC, flashing.color);
		}
	}

	DrawTextEx(hDC, (char *) caption.c_str(), caption.length(), &r, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | DT_END_ELLIPSIS, 0);

	if (buttonType == 0)
		normalFont = (HFONT) SelectObject(hDC, normalFont);
	else if (buttonType == 1)
		activeFont = (HFONT) SelectObject(hDC, activeFont);
	else if (buttonType == 2)
		minimizedFont = (HFONT) SelectObject(hDC, minimizedFont);
	else if (buttonType == 3)
		flashingFont = (HFONT) SelectObject(hDC, flashingFont);

	SetBkMode(hDC, bkMode);
	SetTextColor(hDC, textColor);
}

void ButtonFont::createHandle(FONTSETTINGS processFont, HFONT &hfont)
{
	LOGFONT lf;
	memset(&lf, 0, sizeof(LOGFONT));

	StringCchCopy(lf.lfFaceName, LF_FACESIZE, processFont.name.c_str());
	lf.lfHeight = processFont.height;
	lf.lfItalic = processFont.italic ? TRUE : FALSE;
	lf.lfUnderline = processFont.underline ? TRUE : FALSE;
	lf.lfWeight = processFont.bold ? FW_BOLD : FW_NORMAL;
    lf.lfCharSet = DEFAULT_CHARSET;

	if(hfont != 0)
		DeleteObject(hfont);

	hfont = CreateFontIndirect(&lf);
}

void ButtonFont::drawOutline(HDC destHDC, RECT tmpr, int align, int vertAlign, const string &caption)
{
	OffsetRect(&tmpr, 1, 1);
	DrawTextEx(destHDC, (char *) caption.c_str(), caption.length(), &tmpr, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | DT_END_ELLIPSIS, 0);

	OffsetRect(&tmpr, -1, 0);
	DrawTextEx(destHDC, (char *) caption.c_str(), caption.length(), &tmpr, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | DT_END_ELLIPSIS, 0);

	OffsetRect(&tmpr, -1, 0);
	DrawTextEx(destHDC, (char *) caption.c_str(), caption.length(), &tmpr, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | DT_END_ELLIPSIS, 0);

	OffsetRect(&tmpr, 0, -1);
	DrawTextEx(destHDC, (char *) caption.c_str(), caption.length(), &tmpr, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | DT_END_ELLIPSIS, 0);

	OffsetRect(&tmpr, 0, -1);
	DrawTextEx(destHDC, (char *) caption.c_str(), caption.length(), &tmpr, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | DT_END_ELLIPSIS, 0);

	OffsetRect(&tmpr, 1, 0);
	DrawTextEx(destHDC, (char *) caption.c_str(), caption.length(), &tmpr, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | DT_END_ELLIPSIS, 0);

	OffsetRect(&tmpr, 1, 0);
	DrawTextEx(destHDC, (char *) caption.c_str(), caption.length(), &tmpr, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | DT_END_ELLIPSIS, 0);

	OffsetRect(&tmpr, 0, 1);
	DrawTextEx(destHDC, (char *) caption.c_str(), caption.length(), &tmpr, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | DT_END_ELLIPSIS, 0);
}
