#include "stdafx.h"
#include "ButtonIcon.h"

ButtonIcon::ButtonIcon(const string &name)
{
	defaulticon.iconSize = GetxRCInt(name, "IconSize", 16, 8, 64);
	defaulticon.iconX = GetxRCString(name, "IconX", "0");
	defaulticon.iconY = GetxRCString(name, "IconY", ("-" + IntToString(defaulticon.iconSize/2) + "c") );
	
	defaulticon.icon_saturation = GetxRCInt(name, "IconSaturationIntensity", 255, 0, 255);
	defaulticon.icon_hueintensity = GetxRCInt(name, "IconHueIntensity", 0, 0, 255);
	defaulticon.icon_huecolor = GetxRCColor(name, "IconHueColor", RGB(255, 255, 255));
}

ButtonIcon::~ButtonIcon()
{
}

void ButtonIcon::subconfigure(string type, ICONSETTINGS &workSettings, ICONSETTINGS &defaultSettings)
{
	workSettings.iconSize = GetxRCInt(type, "IconSize", defaultSettings.iconSize, 8, 64);
	workSettings.iconX = GetxRCString(type, "IconX", defaultSettings.iconX);
	workSettings.iconY = GetxRCString(type, "IconY", ("-" + IntToString(workSettings.iconSize/2) + "c"));

	workSettings.icon_saturation = GetxRCInt(type, "IconSaturationIntensity", defaultSettings.icon_saturation, 0, 255);
	workSettings.icon_hueintensity = GetxRCInt(type, "IconHueIntensity", defaultSettings.icon_hueintensity, 0, 255);
	workSettings.icon_huecolor = GetxRCColor(type, "IconHueColor", defaultSettings.icon_huecolor);
}

void ButtonIcon::configure(const string &name)
{
	subconfigure((name + "Normal"), normal, defaulticon);

	subconfigure((name + "Active"), active, normal);

	subconfigure((name + "Minimized"), minimized, normal);

	subconfigure((name + "Flashing"), flashing, normal);
}

void ButtonIcon::apply(HDC hDC, int x, int y, int width, int height, int buttonType, HICON hIcon, bool transparencyMode)
{
	if(hIcon == NULL)
		return;

	int iconX = 0;
	int iconY = 0;
	int iconSize = 16;

	int _saturation = 255;
	int _hueIntensity = 0;
	COLORREF _huecolor = RGB(255, 255, 255);

	if (buttonType == 0)
	{
		iconX = xParseCoord(normal.iconX, 0, width);
		iconY = xParseCoord(normal.iconY, 0, height);

		iconSize = normal.iconSize;
		_saturation = normal.icon_saturation;
		_hueIntensity = normal.icon_hueintensity;
		_huecolor = normal.icon_huecolor;
	}
	else if (buttonType == 1)
	{
		iconX = xParseCoord(active.iconX, 0, width);
		iconY = xParseCoord(active.iconY, 0, height);

		iconSize = active.iconSize;
		_saturation = active.icon_saturation;
		_hueIntensity = active.icon_hueintensity;
		_huecolor = active.icon_huecolor;
	}
	else if (buttonType == 2)
	{
		iconX = xParseCoord(minimized.iconX, 0, width);
		iconY = xParseCoord(minimized.iconY, 0, height);

		iconSize = minimized.iconSize;
		_saturation = minimized.icon_saturation;
		_hueIntensity = minimized.icon_hueintensity;
		_huecolor = minimized.icon_huecolor;
	}
	else if (buttonType == 3)
	{
		iconX = xParseCoord(flashing.iconX, 0, width);
		iconY = xParseCoord(flashing.iconY, 0, height);

		iconSize = flashing.iconSize;
		_saturation = flashing.icon_saturation;
		_hueIntensity = flashing.icon_hueintensity;
		_huecolor = flashing.icon_huecolor;
	}

	if ( (IsOS(OS_XP) && transparencyMode) || _saturation != 255 || _hueIntensity != 0 )
	{
		HDC hdcBuffer = CreateCompatibleDC(hDC);
		HBITMAP hbmBuffer = CreateBitmap(iconSize, iconSize, 1, 32, NULL);
		SelectObject(hdcBuffer, hbmBuffer);

		//Copy RealBG in Icon Colorizing Background
		BitBlt( hdcBuffer, 0, 0, iconSize, iconSize, hDC, x+iconX, y+iconY, SRCCOPY );

		HDC hdcMask = CreateCompatibleDC(hdcBuffer);
		HBITMAP hbmMask = CreateCompatibleBitmap(hdcBuffer, iconSize, iconSize);
		SelectObject(hdcMask, hbmMask);

		// Mask of the icon
		DrawIconEx(hdcMask, 0, 0, hIcon, iconSize, iconSize, 0, NULL, DI_MASK);

		//Remove MagicPink under the Mask (only for XP)
		if ( IsOS(OS_XP) && transparencyMode )
		{
			int iy = 0;
			int ix = 0;
			COLORREF cl = 0;
			BYTE r = 0;
			BYTE g = 0;
			BYTE b = 0;
			for(iy = 0; iy < iconSize; iy++)
			{
				for(ix = 0; ix < iconSize; ix++)
				{
					// Loop through all transparent pixels...
					// If the mask-pixel is white, then break
					if(!GetPixel(hdcMask, ix, iy))
					{
						cl = GetPixel(hdcBuffer, ix, iy);
						r = GetRValue(cl);
						g = GetGValue(cl);
						b = GetBValue(cl);

						if( r == 255 && g == 0 && b == 255 )
						{
							r = 125;
							g = 125;
							b = 125;
							
							SetPixelV(hdcBuffer, ix, iy, RGB(r, g, b));
						}
					}
				}
			}
		}

		//Draw Icon
		DrawIconEx(hdcBuffer, 0, 0, hIcon, iconSize, iconSize, 0, NULL, DI_NORMAL);

		if ( _saturation != 255 || _hueIntensity != 0 )
		{
			int iy = 0;
			int ix = 0;
			COLORREF cl = 0;
			BYTE r = 0;
			BYTE g = 0;
			BYTE b = 0;
			for(iy = 0; iy < iconSize; iy++)
			{
				for(ix = 0; ix < iconSize; ix++)
				{
					// Loop through all transparent pixels...
					// If the mask-pixel is white, then break
					if(!GetPixel(hdcMask, ix, iy))
					{
						cl = GetPixel(hdcBuffer, ix, iy);
						r = GetRValue(cl);
						g = GetGValue(cl);
						b = GetBValue(cl);

						// Saturation effect
						if(_saturation != 255)
						{
							BYTE gray = (BYTE)(r * 0.3086 + g * 0.6094 + b * 0.0820);

							r = (BYTE)((r * _saturation + gray * (255 - _saturation) + 255) >> 8);
							g = (BYTE)((g * _saturation + gray * (255 - _saturation) + 255) >> 8);
							b = (BYTE)((b * _saturation + gray * (255 - _saturation) + 255) >> 8);
						}

						// Hue effect
						if(_hueIntensity != 0 )
						{
							// The B & R values of the hue is swapped here, can somebody tell me why it only works that way?
							// test normal
							r = (BYTE)((r * (255 - _hueIntensity) + GetRValue(_huecolor) * _hueIntensity + 255) >> 8);
							g = (BYTE)((g * (255 - _hueIntensity) + GetGValue(_huecolor) * _hueIntensity + 255) >> 8);
							b = (BYTE)((b * (255 - _hueIntensity) + GetBValue(_huecolor) * _hueIntensity + 255) >> 8);
						}

						SetPixelV(hdcBuffer, ix, iy, RGB(r, g, b));
					}
				}
			}
		}

		BitBlt( hDC, x+iconX, y+iconY, iconSize, iconSize, hdcMask, 0, 0, SRCAND );
		// Combine the foreground with the background
		BitBlt( hDC, x+iconX, y+iconY, iconSize, iconSize, hdcBuffer, 0, 0, SRCPAINT );
			
		hbmMask = (HBITMAP) SelectObject(hdcMask, hbmMask);
		DeleteObject(hbmMask);
		DeleteDC(hdcMask);

		hbmBuffer = (HBITMAP) SelectObject(hdcBuffer, hbmBuffer);
		DeleteObject(hbmBuffer);
		DeleteDC(hdcBuffer);
	}
	else
		DrawIconEx(hDC, x+iconX, y+iconY, hIcon, iconSize, iconSize, 0, NULL, DI_NORMAL);
}

