#if !defined(AFX_BUTTONTEXTURE_H__4F10FDCD_EF77_423F_8223_011704DFFED2__INCLUDED_)
#define AFX_BUTTONTEXTURE_H__4F10FDCD_EF77_423F_8223_011704DFFED2__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

typedef vector<COLORREF> ColorRefList;
typedef vector<COLORREF>::iterator ColorRefListIterator;

#ifndef AC_SRC_ALPHA
#define AC_SRC_ALPHA            0x00000001L
#endif

struct BUTTONSETTINGS
{
	int leftEdge;
	int topEdge;
	int rightEdge;
	int bottomEdge;

	int mode;

	int button_saturation;
	int button_hueIntensity;
	COLORREF button_hueColor;

	int Alpha;

	bool trueTransparent;
};

class ButtonTexture  
{
public:

	ButtonTexture(const string &name);
	virtual ~ButtonTexture();

	void configure(const string &name, const bool &transparencyMode);
	void apply(HDC hDC, int x, int y, int width, int height, int buttonType);

	bool isTransparencyNeeded() { return transparencyNeeded; };

	bool useReallyHover() { return (usehoverSolidColors || (memoryHoverDC != 0 && memoryHoverBitmap != 0)); };
	bool useReallyGrouped() { return (usegroupSolidColors || (memoryGroupDC != 0 && memoryGroupBitmap != 0)); };

	int normalAlpha;
	int activeAlpha;
	int minimizedAlpha;
	int flashingAlpha;
	int hoverAlpha;
	int groupAlpha;

private:

	bool useAlphaMap;

	bool transparencyNeeded;

	void subconfigure(string type, BUTTONSETTINGS &workSettings, BUTTONSETTINGS &defaultSettings);

	void parseCFG(string cfg, COLORREF &bgcolor, COLORREF &lightcolor, COLORREF &darkcolor, int imageType);
	void parseGradientCFG(string cfg, int imageType);
	void solidGradientPaint(HDC destHDC, ColorRefList workGradient, RECT r, bool vertical);
	void paintEffects(HDC destHDC, int width, int height, int button_saturation, int button_hueIntensity, COLORREF button_hueColor );
	void PrepareAlphaChannel(HBITMAP &bmp, int button_saturation, int button_hueIntensity, COLORREF button_hueColor);

	ColorRefList NormalGradient;
	ColorRefList ActiveGradient;
	ColorRefList MinimizedGradient;
	ColorRefList FlashingGradient;

	ColorRefList HoverGradient;
	ColorRefList GroupGradient;

	HDC memoryNormalDC;
	HBITMAP memoryNormalBitmap;

	HDC memoryActiveDC;
	HBITMAP memoryActiveBitmap;

	HDC memoryMinimizedDC;
	HBITMAP memoryMinimizedBitmap;

	HDC memoryFlashingDC;
	HBITMAP memoryFlashingBitmap;

	HDC memoryHoverDC;
	HBITMAP memoryHoverBitmap;

	HDC memoryGroupDC;
	HBITMAP memoryGroupBitmap;

	int imageNormalHeight;
	int imageNormalWidth;

	int imageActiveHeight;
	int imageActiveWidth;

	int imageMinimizedHeight;
	int imageMinimizedWidth;

	int imageFlashingHeight;
	int imageFlashingWidth;

	int imageHoverHeight;
	int imageHoverWidth;

	int imageGroupHeight;
	int imageGroupWidth;

	BUTTONSETTINGS defaultbutton;;
	BUTTONSETTINGS normal;
	BUTTONSETTINGS active;
	BUTTONSETTINGS minimized;
	BUTTONSETTINGS flashing;
	BUTTONSETTINGS hover;	
	BUTTONSETTINGS group;

	string defaultsolidcolors;
	int defaultbevelSize;
	string defaultgradientColor;
	bool defaultverticalGradient;

	bool usenormalSolidColors;
	string normalsolidcolors;
	bool normalverticalGradient;
	int normalbevelSize;
	string normalgradientColor;
	COLORREF normalbackgroundColor;
	COLORREF normaldarkBevelColor;
	COLORREF normallightBevelColor;

	bool useactiveSolidColors;
	string activesolidcolors;
	bool activeverticalGradient;
	int activebevelSize;
	string activegradientColor;
	COLORREF activebackgroundColor;
	COLORREF activedarkBevelColor;
	COLORREF activelightBevelColor;

	bool useminimizedSolidColors;
	bool minimizedverticalGradient;
	int minimizedbevelSize;
	COLORREF minimizedbackgroundColor;
	COLORREF minimizeddarkBevelColor;
	COLORREF minimizedlightBevelColor;

	bool useflashingSolidColors;
	bool flashingverticalGradient;
	int flashingbevelSize;
	COLORREF flashingbackgroundColor;
	COLORREF flashingdarkBevelColor;
	COLORREF flashinglightBevelColor;

	bool usehoverSolidColors;
	bool hoververticalGradient;
	int hoverbevelSize;
	COLORREF hoverbackgroundColor;
	COLORREF hoverdarkBevelColor;
	COLORREF hoverlightBevelColor;

	bool usegroupSolidColors;
	bool groupverticalGradient;
	int groupbevelSize;
	COLORREF groupbackgroundColor;
	COLORREF groupdarkBevelColor;
	COLORREF grouplightBevelColor;
};

#endif // !defined(AFX_BUTTONTEXTURE_H__4F10FDCD_EF77_423F_8223_011704DFFED2__INCLUDED_)
