#include "stdafx.h"
#include "TaskbarButton.h"
#include "Taskbar.h"
#include "WindowUtilities.h"
#include "ButtonTexture.h"
#include "ButtonIcon.h"
#include "ButtonFont.h"

extern vector<string> CustomIconList;

//Grouping
#define	X_GROUP_MINIMIZE	10
#define	X_GROUP_RESTORE		11
#define	X_GROUP_MAXIMIZE	12
#define	X_GROUP_CLOSE		13

//#define	X_GROUP_GROUP		14
//#define	X_GROUP_UNGROUP		15

#define	X_GROUP_SEPARATOR	66

TaskbarButton::TaskbarButton( Taskbar *actualtaskbar, HWND hWnd ):taskbar(NULL), buttonx(0), buttony(0), buttonheight(0), buttonwidth(0)
{
	hwndgrouplist.clear();
	
	useHover = false;
	flashNow = false;

	currentState = STATE_NORMAL;

	taskbar = actualtaskbar;

	hButtonTask = hWnd;

	classname = GetTaskFileName( hWnd );

	hwndgrouplist.insert(hwndgrouplist.end(), hWnd);

	reallyuseGrouped = taskbar->buttontexture->useReallyGrouped();
	reallyuseHover = taskbar->buttontexture->useReallyHover();

	position = -1;

	icon = NULL;

	if(taskbar->showText || taskbar->showToolTip)
		caption = GetWindowCaption();
	else
		caption = "";
	
	if (taskbar->showToolTip)
		tooltip = caption;
}

TaskbarButton::~TaskbarButton()
{
	hwndgrouplist.clear();

	if (icon)
		DestroyIcon(icon);
}

BOOL TaskbarButton::HasPoint( int xP, int yP )
{
	if ( xP < (buttonx + buttonwidth) && xP >= buttonx )
	{
		if ( yP < (buttony + buttonheight) && yP >= buttony )
			return TRUE;
	}

	return FALSE;
}

BOOL TaskbarButton::onUpdate(bool needRedraw)
{
	if (taskbar->showText || taskbar->showToolTip)
	{
		string temp1 = caption;
		string temp2 = GetWindowCaption();

		if ( stricmp( temp1.c_str(), temp2.c_str() ) != NULL && !temp2.empty() )
		{
			caption = temp2;
			if (taskbar->showToolTip && hwndgrouplist.size() > 1)
			{
				temp2.append( " - Grouped (" );
				temp2.append( IntToString(hwndgrouplist.size()) );
				temp2.append( ")" );
				tooltip = temp2;
			}
			else if (taskbar->showToolTip)
			{	
				tooltip = temp2;
			}
			if (taskbar->showText)
				needRedraw = true;
		}
	}

	if (taskbar->displayButtonType >= 0)
		needRedraw = true;

	return needRedraw;	
}

string TaskbarButton::GetWindowCaption( HWND check_hTask )
{
	if (check_hTask == NULL)
		check_hTask = hButtonTask;
	
	return GetTaskWindowText(check_hTask);
}

void TaskbarButton::onButtonClick( int x, int y, int action, bool minimize )
{
	switch (action)
	{
		case 1:
			
			Select(minimize);
		    break;

		case 2:

			Menu();
		    break;

		case 3:

			Close(hButtonTask);
		    break;

		case 4:

			OnTopToggle(hButtonTask);
			break;
	}
}

void TaskbarButton::Select(bool minimize)
{
	if( flashNow )
	{
		SetFlashing(FALSE);
		currentState = getCurrentState();
	}

	if (hwndgrouplist.size() > 1 && taskbar->useGrouping)
	{
		GroupMenu();
		return;
	}

	if ( minimize )
	{
		if (!taskbar->noMinimizeOnClick)
			MinimizeWindow(hButtonTask);
		else
			SelectWindow(hButtonTask);
	}
	else
	{
		switch(currentState)
		{
			case STATE_SELECTED:

				if (!taskbar->noMinimizeOnClick)
					MinimizeWindow(hButtonTask);
				else
					SelectWindow(hButtonTask);
				break;

			default:

				SelectWindow(hButtonTask);
				break;
		}
	}
}

void TaskbarButton::Menu()
{
	HMENU systemMenu;

	systemMenu = GetSystemMenu( hButtonTask, FALSE );

	WINDOWPLACEMENT wp;
	wp.length = sizeof(WINDOWPLACEMENT);
	GetWindowPlacement( hButtonTask, &wp );

	// restore is enabled only if minimized or maximized (not normal)
	EnableMenuItem( systemMenu, SC_RESTORE, MF_BYCOMMAND | (wp.showCmd != SW_SHOWNORMAL ? MF_ENABLED : MF_GRAYED) );

	// move is enabled only if normal
	EnableMenuItem( systemMenu, SC_MOVE, MF_BYCOMMAND | (wp.showCmd == SW_SHOWNORMAL ? MF_ENABLED : MF_GRAYED) );

	// size is enabled only if normal
	EnableMenuItem( systemMenu, SC_SIZE, MF_BYCOMMAND | (wp.showCmd == SW_SHOWNORMAL ? MF_ENABLED : MF_GRAYED) );

	// minimize is enabled only if not minimized
	EnableMenuItem( systemMenu, SC_MOVE, MF_BYCOMMAND | (wp.showCmd != SW_SHOWMINIMIZED ? MF_ENABLED : MF_GRAYED) );

	// maximize is enabled only if not maximized
	EnableMenuItem( systemMenu, SC_MOVE, MF_BYCOMMAND | (wp.showCmd != SW_SHOWMAXIMIZED ? MF_ENABLED : MF_GRAYED) );

	int counter = 0;

	if (hwndgrouplist.size() > 1 && taskbar->useGrouping)
	{	
		AppendMenu( systemMenu, MF_SEPARATOR, X_GROUP_SEPARATOR, "" );

		AppendMenu( systemMenu, MF_STRING, X_GROUP_MINIMIZE, "Minimize All In Group" );
		AppendMenu( systemMenu, MF_STRING, X_GROUP_RESTORE, "Restore All In Group" );
		AppendMenu( systemMenu, MF_STRING, X_GROUP_MAXIMIZE, "Maximize All In Group" );
		AppendMenu( systemMenu, MF_STRING, X_GROUP_CLOSE, "Close All In Group" );

		AppendMenu( systemMenu, MF_SEPARATOR, X_GROUP_SEPARATOR, "" );

		counter = 1;
		for(hWndGroupListIterator hglit = hwndgrouplist.begin(); hglit != hwndgrouplist.end(); ++hglit)
		{
			AppendMenu( systemMenu, MF_STRING , counter, ("Close " + GetWindowCaption(*hglit)).c_str() );
			counter++;
		}
		counter--;
		//END Grouping
	}

	// let application modify menu
	PostMessage( hButtonTask, WM_INITMENUPOPUP, (WPARAM) systemMenu, MAKELPARAM( 0, TRUE ) );
	PostMessage( hButtonTask, WM_INITMENU, (WPARAM) systemMenu, 0 );

	// display the menu
	POINT pt;
	GetCursorPos( &pt );

	int command = TrackPopupMenu( systemMenu,
	                              TPM_RETURNCMD | TPM_RIGHTBUTTON,
	                              pt.x, pt.y, 0,
	                              taskbar->gethWnd(),
	                              NULL );

	if (hwndgrouplist.size() > 1 && taskbar->useGrouping)
	{
		DeleteMenu( systemMenu, X_GROUP_SEPARATOR, MF_SEPARATOR );

		DeleteMenu( systemMenu, X_GROUP_SEPARATOR, MF_BYCOMMAND );
		DeleteMenu( systemMenu, X_GROUP_MINIMIZE, MF_BYCOMMAND );
		DeleteMenu( systemMenu, X_GROUP_RESTORE, MF_BYCOMMAND );
		DeleteMenu( systemMenu, X_GROUP_MAXIMIZE, MF_BYCOMMAND );
		DeleteMenu( systemMenu, X_GROUP_CLOSE, MF_BYCOMMAND );

		DeleteMenu( systemMenu, X_GROUP_SEPARATOR, MF_SEPARATOR );

		while ( counter >= 1 )
		{
			DeleteMenu( systemMenu, counter, MF_BYCOMMAND );
			counter--;
		}
	}

	if ( command )
	{
		if (hwndgrouplist.size() > 1 && taskbar->useGrouping)
		{
			switch( command )
			{
				case X_GROUP_MINIMIZE:
				{
					for ( hWndGroupListIterator hglit = hwndgrouplist.begin(); hglit != hwndgrouplist.end(); ++hglit )
						MinimizeWindow( (*hglit) );
					
					return;
				}

				case X_GROUP_RESTORE:
				{
					for ( hWndGroupListIterator hglit = hwndgrouplist.begin(); hglit != hwndgrouplist.end(); ++hglit )
						RestoreWindow( (*hglit) );

					return;
				}

				case X_GROUP_MAXIMIZE:
				{
					for ( hWndGroupListIterator hglit = hwndgrouplist.begin(); hglit != hwndgrouplist.end(); ++hglit )
						MaximizeWindow( (*hglit) );

					return;
				}

				case X_GROUP_CLOSE:
				{
					for ( hWndGroupListIterator hglit = hwndgrouplist.begin(); hglit != hwndgrouplist.end(); ++hglit )
						Close( (*hglit) );
						
					return;
				}
			}

			hWndGroupListIterator hglit = hwndgrouplist.begin();
			counter = 1; 
			while (counter < command)
			{
				hglit++;
				counter++;
			}

			if ( command <= hwndgrouplist.size() )
			{
				Close( (*hglit) );
				hButtonTask = (*hglit);
				return;
			}
		}
		
		if ( command != SC_MINIMIZE && command != SC_CLOSE )
			SelectWindow(hButtonTask);

		PostMessage( hButtonTask, WM_SYSCOMMAND, (WPARAM) command, MAKELPARAM( pt.x, pt.y ) );
	}
}

void TaskbarButton::GroupMenu()
{
	HMENU GroupMenu = CreatePopupMenu();

	int counter = 1;
	for(hWndGroupListIterator hglit = hwndgrouplist.begin(); hglit != hwndgrouplist.end(); ++hglit)
	{
		AppendMenu( GroupMenu, MF_STRING, counter, GetWindowCaption((*hglit)).c_str() );
		counter++;
	}

	// display the menu
	POINT pt;
	GetCursorPos( &pt );

	int command = TrackPopupMenu( GroupMenu,
	                              TPM_RETURNCMD | TPM_RIGHTBUTTON,
	                              pt.x, pt.y, 0,
	                              taskbar->gethWnd(),
	                              NULL );

	if ( command )
	{
		hWndGroupListIterator hglit = hwndgrouplist.begin();
		counter = 1; 
		while (counter < command)
		{
			hglit++;
			counter++;
		}

		if ( counter <= hwndgrouplist.size() )
		{
			SelectWindow( (*hglit) );
			hButtonTask = (*hglit);
		}
	}
	
	if (GroupMenu)
		DestroyMenu(GroupMenu);
}

int TaskbarButton::getCurrentState()
{
	return GetState(hButtonTask);
}

void TaskbarButton::setButtonLayout( int x, int y, int width, int height )
{
	buttonx = x;
	buttony = y;
	buttonwidth = width;	
	buttonheight = height;
}

void TaskbarButton::onPaint( HDC hDC )
{
	//Update Icon - Sure is Sure
	if ( taskbar->showIcon )
	{
		if (icon)
		{
			DestroyIcon(icon);
			icon = NULL;
		}

		if ( GetIconOnCustomList( hButtonTask, icon, CustomIconList, taskbar->useBigIcon) )
			icon = (HICON)CopyImage((HANDLE)icon, IMAGE_ICON, 0, 0, 0);
	}

	if (!flashNow)
	{
		currentState = getCurrentState();

		if( STATE_SELECTED == currentState )
		{
			taskbar->lastActive = hButtonTask;
		}
	}

	if ( flashNow && STATE_FLASHING != currentState )
		currentState = STATE_FLASHING;
	else if ( flashNow && STATE_FLASHING == currentState )
		currentState = STATE_NORMAL;

	//if (IsOS(OS_2KXP))
	//{
		HDC tmpDC = CreateCompatibleDC(hDC);
		HBITMAP tmpBmp = CreateBitmap(buttonwidth, buttonheight, 1, 32, NULL);
		SelectObject(tmpDC, tmpBmp);
	//}

	//Button Background Painting with Icon and Text 
	if(currentState == STATE_FLASHING)
	{
		if (taskbar->useGrouping && hwndgrouplist.size() > 1 && reallyuseGrouped && !(reallyuseHover && useHover))
		{
			if (taskbar->buttontexture->groupAlpha != 255 && IsOS(OS_2KXP))
			{
				BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, taskbar->buttontexture->groupAlpha, /*(taskbar->buttontexture->has_groupAlphaMap ? AC_SRC_ALPHA :*/ 0/*)*/ };

				taskbar->buttontexture->apply(tmpDC, 0, 0, buttonwidth, buttonheight, 5);
				AlphaBlend(hDC, buttonx, buttony, buttonwidth, buttonheight, tmpDC, 0, 0, buttonwidth, buttonheight, blendFunction);
			}
			else
				taskbar->buttontexture->apply(hDC, buttonx, buttony, buttonwidth, buttonheight, 5);
		}
		else if (reallyuseHover && useHover)
		{
			if (taskbar->buttontexture->hoverAlpha != 255 && IsOS(OS_2KXP))
			{
				BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, taskbar->buttontexture->hoverAlpha, /*(taskbar->buttontexture->has_hoverAlphaMap ? AC_SRC_ALPHA :*/ 0/*)*/ };

				taskbar->buttontexture->apply(tmpDC, 0, 0, buttonwidth, buttonheight, 4);
				AlphaBlend(hDC, buttonx, buttony, buttonwidth, buttonheight, tmpDC, 0, 0, buttonwidth, buttonheight, blendFunction);
			}
			else
				taskbar->buttontexture->apply(hDC, buttonx, buttony, buttonwidth, buttonheight, 4);
		}
		else
		{
			if (taskbar->buttontexture->flashingAlpha != 255 && IsOS(OS_2KXP))
			{
				BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, taskbar->buttontexture->flashingAlpha , /*(taskbar->buttontexture->has_flashingAlphaMap ? AC_SRC_ALPHA :*/ 0/*)*/ };

				taskbar->buttontexture->apply(tmpDC, 0, 0, buttonwidth, buttonheight, 3);
				AlphaBlend(hDC, buttonx, buttony, buttonwidth, buttonheight, tmpDC, 0, 0, buttonwidth, buttonheight, blendFunction);
			}
			else
				taskbar->buttontexture->apply(hDC, buttonx, buttony, buttonwidth, buttonheight, 3);
		}

		if(taskbar->showIcon)
		{
			taskbar->buttonicon->apply(hDC, buttonx, buttony, buttonwidth, buttonheight, 3, icon, taskbar->transparencyMode);
		}
		if (taskbar->showText)
		{
			CString text = caption.c_str();
			if (hwndgrouplist.size() > 1 && taskbar->useGrouping)
			{
				int found = text.Find(" - ");
				if ( found != -1 && text.Find(" - ", found+3) != -1 )
					text = text.Mid(found+3, text.Find(" - ", found+3)-found-3);
				else if ( found != -1 )
					text = text.Left(text.Find(" - "));
				text += " ( ";
				text += IntToString( hwndgrouplist.size() ).c_str();
				text += " )";
			}
				
			taskbar->buttonfont->apply(hDC, buttonx, buttony, buttonwidth, buttonheight, (string) text, 3);
		}
	}
	else if (currentState == STATE_SELECTED)
	{
		if (taskbar->useGrouping && hwndgrouplist.size() > 1 && reallyuseGrouped && !(reallyuseHover && useHover))
		{
			if (taskbar->buttontexture->groupAlpha != 255 && IsOS(OS_2KXP))
			{
				BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, taskbar->buttontexture->groupAlpha, /*(taskbar->buttontexture->has_groupAlphaMap ? AC_SRC_ALPHA :*/ 0/*)*/ };

				taskbar->buttontexture->apply(tmpDC, 0, 0, buttonwidth, buttonheight, 5);
				AlphaBlend(hDC, buttonx, buttony, buttonwidth, buttonheight, tmpDC, 0, 0, buttonwidth, buttonheight, blendFunction);
			}
			else
				taskbar->buttontexture->apply(hDC, buttonx, buttony, buttonwidth, buttonheight, 5);
		}
		else if (reallyuseHover && useHover)
		{
			if (taskbar->buttontexture->hoverAlpha != 255 && IsOS(OS_2KXP))
			{
				BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, taskbar->buttontexture->hoverAlpha, /*(taskbar->buttontexture->has_hoverAlphaMap ? AC_SRC_ALPHA :*/ 0/*)*/ };

				taskbar->buttontexture->apply(tmpDC, 0, 0, buttonwidth, buttonheight, 4);
				AlphaBlend(hDC, buttonx, buttony, buttonwidth, buttonheight, tmpDC, 0, 0, buttonwidth, buttonheight, blendFunction);
			}
			else
				taskbar->buttontexture->apply(hDC, buttonx, buttony, buttonwidth, buttonheight, 4);
		}
		else
		{
			if (taskbar->buttontexture->activeAlpha != 255 && IsOS(OS_2KXP))
			{
				BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, taskbar->buttontexture->activeAlpha , /*(taskbar->buttontexture->has_activeAlphaMap ? AC_SRC_ALPHA :*/ 0/*)*/ };

				taskbar->buttontexture->apply(tmpDC, 0, 0, buttonwidth, buttonheight, 1);
				AlphaBlend(hDC, buttonx, buttony, buttonwidth, buttonheight, tmpDC, 0, 0, buttonwidth, buttonheight, blendFunction);
			}
			else
				taskbar->buttontexture->apply(hDC, buttonx, buttony, buttonwidth, buttonheight, 1);
		}

		if(taskbar->showIcon)
		{
			taskbar->buttonicon->apply(hDC, buttonx, buttony, buttonwidth, buttonheight, 1, icon, taskbar->transparencyMode);
		}
		if (taskbar->showText)
		{
			CString text = caption.c_str();
			if (hwndgrouplist.size() > 1 && taskbar->useGrouping)
			{
				int found = text.Find(" - ");
				if ( found != -1 && text.Find(" - ", found+3) != -1)
					text = text.Mid(found+3, text.Find(" - ", found+3)-found-3);
				else if ( found != -1 )
					text = text.Left(text.Find(" - "));
				text += " ( ";
				text += IntToString( hwndgrouplist.size() ).c_str();
				text += " )";
			}

			taskbar->buttonfont->apply(hDC, buttonx, buttony, buttonwidth, buttonheight, (string) text, 1);
		}
	}
	else if (currentState == STATE_MINIMIZED)
	{
		if (taskbar->useGrouping && hwndgrouplist.size() > 1 && reallyuseGrouped && !(reallyuseHover && useHover))
		{
			if (taskbar->buttontexture->groupAlpha != 255 && IsOS(OS_2KXP))
			{
				BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, taskbar->buttontexture->groupAlpha, /*(taskbar->buttontexture->has_groupAlphaMap ? AC_SRC_ALPHA :*/ 0/*)*/ };

				taskbar->buttontexture->apply(tmpDC, 0, 0, buttonwidth, buttonheight, 5);
				AlphaBlend(hDC, buttonx, buttony, buttonwidth, buttonheight, tmpDC, 0, 0, buttonwidth, buttonheight, blendFunction);
			}
			else
				taskbar->buttontexture->apply(hDC, buttonx, buttony, buttonwidth, buttonheight, 5);
		}
		else if (reallyuseHover && useHover)
		{
			if (taskbar->buttontexture->hoverAlpha != 255 && IsOS(OS_2KXP))
			{
				BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, taskbar->buttontexture->hoverAlpha, /*(taskbar->buttontexture->has_hoverAlphaMap ? AC_SRC_ALPHA :*/ 0/*)*/ };

				taskbar->buttontexture->apply(tmpDC, 0, 0, buttonwidth, buttonheight, 4);
				AlphaBlend(hDC, buttonx, buttony, buttonwidth, buttonheight, tmpDC, 0, 0, buttonwidth, buttonheight, blendFunction);
			}
			else
				taskbar->buttontexture->apply(hDC, buttonx, buttony, buttonwidth, buttonheight, 4);
		}
		else
		{
			if (taskbar->buttontexture->minimizedAlpha != 255 && IsOS(OS_2KXP))
			{
				BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, taskbar->buttontexture->minimizedAlpha , /*(taskbar->buttontexture->has_minimizedAlphaMap ? AC_SRC_ALPHA :*/ 0/*)*/ };

				taskbar->buttontexture->apply(tmpDC, 0, 0, buttonwidth, buttonheight, 2);
				AlphaBlend(hDC, buttonx, buttony, buttonwidth, buttonheight, tmpDC, 0, 0, buttonwidth, buttonheight, blendFunction);
			}
			else
				taskbar->buttontexture->apply(hDC, buttonx, buttony, buttonwidth, buttonheight, 2);
		}

		if(taskbar->showIcon)
		{
			taskbar->buttonicon->apply(hDC, buttonx, buttony, buttonwidth, buttonheight, 2, icon, taskbar->transparencyMode);
		}
		if (taskbar->showText)
		{
			CString text = caption.c_str();
			if (hwndgrouplist.size() > 1 && taskbar->useGrouping)
			{
				int found = text.Find(" - ");
				if ( found != -1 && text.Find(" - ", found+3) != -1)
					text = text.Mid(found+3, text.Find(" - ", found+3)-found-3);
				else if ( found != -1 )
					text = text.Left(text.Find(" - "));
				text += " ( ";
				text += IntToString( hwndgrouplist.size() ).c_str();
				text += " )";
			}

			taskbar->buttonfont->apply(hDC, buttonx, buttony, buttonwidth, buttonheight, (string) text, 2);
		}
	}
	else
	{
		if (taskbar->useGrouping && hwndgrouplist.size() > 1 && reallyuseGrouped && !(reallyuseHover && useHover))
		{
			if (taskbar->buttontexture->groupAlpha != 255 && IsOS(OS_2KXP))
			{
				BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, taskbar->buttontexture->groupAlpha, /*(taskbar->buttontexture->has_groupAlphaMap ? AC_SRC_ALPHA :*/ 0/*)*/ };

				taskbar->buttontexture->apply(tmpDC, 0, 0, buttonwidth, buttonheight, 5);
				AlphaBlend(hDC, buttonx, buttony, buttonwidth, buttonheight, tmpDC, 0, 0, buttonwidth, buttonheight, blendFunction);
			}
			else
				taskbar->buttontexture->apply(hDC, buttonx, buttony, buttonwidth, buttonheight, 5);
		}
		else if (reallyuseHover && useHover)
		{
			if (taskbar->buttontexture->hoverAlpha != 255 && IsOS(OS_2KXP))
			{
				BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, taskbar->buttontexture->hoverAlpha, /*(taskbar->buttontexture->has_hoverAlphaMap ? AC_SRC_ALPHA :*/ 0/*)*/ };

				taskbar->buttontexture->apply(tmpDC, 0, 0, buttonwidth, buttonheight, 4);
				AlphaBlend(hDC, buttonx, buttony, buttonwidth, buttonheight, tmpDC, 0, 0, buttonwidth, buttonheight, blendFunction);
			}
			else
				taskbar->buttontexture->apply(hDC, buttonx, buttony, buttonwidth, buttonheight, 4);
		}
		else
		{
			if (taskbar->buttontexture->normalAlpha != 255 && IsOS(OS_2KXP))
			{
				BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, taskbar->buttontexture->normalAlpha, /*(taskbar->buttontexture->has_normalAlphaMap ? AC_SRC_ALPHA :*/ 0/*)*/ };

				taskbar->buttontexture->apply(tmpDC, 0, 0, buttonwidth, buttonheight, 0);
				AlphaBlend(hDC, buttonx, buttony, buttonwidth, buttonheight, tmpDC, 0, 0, buttonwidth, buttonheight, blendFunction);
			}
			else
				taskbar->buttontexture->apply(hDC, buttonx, buttony, buttonwidth, buttonheight, 0);
		}

		if(taskbar->showIcon)
		{
			taskbar->buttonicon->apply(hDC, buttonx, buttony, buttonwidth, buttonheight, 0, icon, taskbar->transparencyMode);
		}
		if (taskbar->showText)
		{
			CString text = caption.c_str();
			if (hwndgrouplist.size() > 1 && taskbar->useGrouping)
			{
				int found = text.Find(" - ");
				if ( found != -1 && text.Find(" - ", found+3) != -1)
					text = text.Mid(found+3, text.Find(" - ", found+3)-found-3);
				else if ( found != -1 )
					text = text.Left(text.Find(" - "));
				text += " ( ";
				text += IntToString( hwndgrouplist.size() ).c_str();
				text += " )";
			}

			taskbar->buttonfont->apply(hDC, buttonx, buttony, buttonwidth, buttonheight, (string) text, 0);
		}
	}

	//if (IsOS(OS_2KXP))
	//{
		tmpBmp = (HBITMAP) SelectObject(tmpDC, tmpBmp);
		DeleteObject(tmpBmp);
		DeleteDC(tmpDC);
	//}
}
